package org.xbib.event.async.impl;

import org.xbib.event.loop.thread.FastThreadLocalThread;
import org.xbib.event.loop.thread.ThreadInfo;

import java.util.concurrent.TimeUnit;

public class AsyncThread extends FastThreadLocalThread {

    private final boolean worker;
    final ThreadInfo info;
    ContextInternal context;
    ClassLoader topLevelTCCL;

    public AsyncThread(Runnable target, String name, boolean worker, long maxExecTime, TimeUnit maxExecTimeUnit) {
        super(target, name);
        this.worker = worker;
        this.info = new ThreadInfo(maxExecTimeUnit, maxExecTime);
    }

    /**
     * @return the current context of this thread, this method must be called from the current thread
     */
    ContextInternal context() {
        return context;
    }

    void executeStart() {
        if (context == null) {
            info.startTime = System.nanoTime();
        }
    }

    void executeEnd() {
        if (context == null) {
            info.startTime = 0;
        }
    }

    public long startTime() {
        return info.startTime;
    }

    public boolean isWorker() {
        return worker;
    }

    public long maxExecTime() {
        return info.maxExecTime;
    }

    public TimeUnit maxExecTimeUnit() {
        return info.maxExecTimeUnit;
    }

}
