package org.xbib.event.async.yield.primitives.dbl.ops;

import org.xbib.event.async.yield.Yield;
import org.xbib.event.async.yield.primitives.dbl.DoubleAdvancer;
import org.xbib.event.async.yield.primitives.dbl.DoubleQuery;
import org.xbib.event.async.yield.primitives.dbl.DoubleTraverser;
import org.xbib.event.async.yield.primitives.dbl.DoubleYield;

import java.util.function.DoublePredicate;

public class DoubleTakeWhile implements DoubleAdvancer, DoubleTraverser {

    private final DoubleQuery upstream;

    private final DoublePredicate predicate;

    private boolean hasNext;

    public DoubleTakeWhile(DoubleQuery upstream, DoublePredicate predicate) {
        this.upstream = upstream;
        this.predicate = predicate;
        this.hasNext = true;
    }

    @Override
    public void traverse(DoubleYield yield) {
        upstream.shortCircuit(item -> {
            if (!predicate.test(item)) {
                Yield.bye();
            }
            yield.ret(item);
        });
    }

    @Override
    public boolean tryAdvance(DoubleYield yield) {
        if (!hasNext) return false; // Once predicate is false it finishes the iteration
        DoubleYield takeWhile = item -> {
            if (predicate.test(item)) {
                yield.ret(item);
            } else {
                hasNext = false;
            }
        };
        return upstream.tryAdvance(takeWhile) && hasNext;
    }
}
