package org.xbib.event.clock;

import java.util.Objects;
import java.util.concurrent.Callable;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.xbib.event.Event;
import org.xbib.event.common.EventManager;

public class ClockEventService implements Callable<Boolean> {

    private static final Logger logger = Logger.getLogger(ClockEventService.class.getName());

    private final ClockEventManagerService clockEventManagerService;

    private final String name;

    private final String eventType;

    public ClockEventService(ClockEventManagerService clockEventManagerService,
                             String name,
                             String eventType) {
        this.clockEventManagerService = clockEventManagerService;
        this.name = name;
        this.eventType = Objects.requireNonNull(eventType, "clock event type must not be null");
    }

    @Override
    public Boolean call() {
        try {
            if (clockEventManagerService.getSuspended().contains(name)) {
                logger.log(Level.FINE, "clock event service " + name + " is suspended, unable to continue");
                return false;
            } else {
                Event clockEvent = EventManager.eventBuilder()
                        .setType(eventType)

                        .build();
                clockEventManagerService.publish(clockEvent);
                return true;
            }
        } catch (Throwable t) {
            logger.log(Level.WARNING, t.getMessage(), t);
            return false;
        }
    }
}
