package org.xbib.event.log;

import java.io.IOException;
import java.time.Instant;
import java.util.Objects;
import java.util.logging.Level;
import java.util.logging.Logger;

public class SizeBasedRotationPolicy implements RotationPolicy {

    private static final Logger logger = Logger.getLogger(SizeBasedRotationPolicy.class.getName());

    private final long maxByteCount;

    private Rotatable rotatable;

    public SizeBasedRotationPolicy(long maxByteCount) {
        if (maxByteCount < 1) {
            String message = String.format("invalid size {maxByteCount=%d}", maxByteCount);
            throw new IllegalArgumentException(message);
        }
        this.maxByteCount = maxByteCount;
    }

    public long getMaxByteCount() {
        return maxByteCount;
    }

    @Override
    public boolean isWriteSensitive() {
        return true;
    }

    @Override
    public void acceptWrite(long byteCount) {
        if (byteCount > maxByteCount) {
            Instant instant =  rotatable.getConfig().getClock().now();
            try {
                rotatable.rotate(this, instant);
            } catch (IOException e) {
                logger.log(Level.SEVERE, e.getMessage(), e);
                rotatable.onException(this, instant, rotatable.getConfig().getPath(), e);
            }
        }
    }

    @Override
    public void start(Rotatable rotatable) {
        this.rotatable = rotatable;
    }

    @Override
    public boolean equals(Object instance) {
        if (this == instance) return true;
        if (instance == null || getClass() != instance.getClass()) return false;
        SizeBasedRotationPolicy that = (SizeBasedRotationPolicy) instance;
        return maxByteCount == that.maxByteCount;
    }

    @Override
    public int hashCode() {
        return Objects.hash(maxByteCount);
    }

    @Override
    public String toString() {
        return String.format("SizeBasedRotationPolicy{maxByteCount=%d}", maxByteCount);
    }
}
