package org.xbib.event.log;

import java.time.Duration;
import java.time.Instant;
import java.util.concurrent.TimeUnit;
import java.util.logging.Level;
import java.util.logging.Logger;

public abstract class TimeBasedRotationPolicy implements RotationPolicy {

    private static final Logger logger = Logger.getLogger(TimeBasedRotationPolicy.class.getName());

    public TimeBasedRotationPolicy() {
    }

    @Override
    public boolean isWriteSensitive() {
        return false;
    }

    @Override
    public void acceptWrite(long byteCount) {
        throw new UnsupportedOperationException();
    }

    @Override
    public void start(Rotatable rotatable) {
        RotationConfig config = rotatable.getConfig();
        Clock clock = config.getClock();
        Instant currentInstant = clock.now();
        Instant triggerInstant = getTriggerInstant(clock);
        long triggerDelayMillis = Duration.between(currentInstant, triggerInstant).toMillis();
        Runnable task = () -> {
            try {
                rotatable.rotate(TimeBasedRotationPolicy.this, triggerInstant);
                start(rotatable);
            } catch (Exception e) {
                logger.log(Level.SEVERE, e.getMessage(), e);
                rotatable.onException(this, currentInstant, rotatable.getConfig().getPath(), e);
            }
        };
        config.getScheduler().schedule(task, triggerDelayMillis, TimeUnit.MILLISECONDS);
    }

    protected abstract Instant getTriggerInstant(Clock clock);
}
