package org.xbib.event.wal;

import java.io.IOException;
import java.io.InputStream;

public class ByteCountingInputStream extends InputStream {

    private final InputStream in;
    private long bytesRead = 0L;
    private long bytesSkipped = 0L;

    private long bytesReadSinceMark = 0L;
    private long bytesSkippedSinceMark = 0L;

    public ByteCountingInputStream(final InputStream in) {
        this.in = in;
    }

    public ByteCountingInputStream(final InputStream in, final long initialOffset) {
        this.in = in;
        this.bytesSkipped = initialOffset;
    }

    @Override
    public int read() throws IOException {
        final int fromSuper = in.read();
        if (fromSuper >= 0) {
            bytesRead++;
            bytesReadSinceMark++;
        }
        return fromSuper;
    }

    @Override
    public int read(byte[] b, int off, int len) throws IOException {
        final int fromSuper = in.read(b, off, len);
        if (fromSuper >= 0) {
            bytesRead += fromSuper;
            bytesReadSinceMark += fromSuper;
        }

        return fromSuper;
    }

    @Override
    public int read(byte[] b) throws IOException {
        return read(b, 0, b.length);
    }

    @Override
    public long skip(final long n) throws IOException {
        final long skipped = in.skip(n);
        if (skipped >= 0) {
            bytesSkipped += skipped;
            bytesSkippedSinceMark += skipped;
        }
        return skipped;
    }

    public long getBytesRead() {
        return bytesRead;
    }

    public long getBytesSkipped() {
        return bytesSkipped;
    }

    public long getBytesConsumed() {
        return getBytesRead() + getBytesSkipped();
    }

    @Override
    public void mark(final int readlimit) {
        in.mark(readlimit);

        bytesReadSinceMark = 0L;
        bytesSkippedSinceMark = 0L;
    }

    @Override
    public boolean markSupported() {
        return in.markSupported();
    }

    @Override
    public void reset() throws IOException {
        in.reset();
        bytesRead -= bytesReadSinceMark;
        bytesSkipped -= bytesSkippedSinceMark;

        bytesReadSinceMark = 0L;
        bytesSkippedSinceMark = 0L;
    }

    @Override
    public void close() throws IOException {
        in.close();
    }

    @Override
    public int available() throws IOException {
        return in.available();
    }
}
