package org.xbib.event.wal;

import java.io.Closeable;
import java.io.IOException;
import java.util.Collection;
import java.util.Map;
import java.util.Set;

public interface WriteAheadJournal<T> extends Closeable {

    JournalRecovery recoverRecords(Map<Object, T> recordMap, Set<String> swapLocations) throws IOException;

    /**
     * Updates the journal with the given set of records
     *
     * @param records the records to update
     * @param recordLookup a lookup that can be used to access the current value of a record, given its ID
     *
     * @throws IOException if unable to write to the underlying storage mechanism
     */
    void update(Collection<T> records, RecordLookup<T> recordLookup) throws IOException;

    void writeHeader() throws IOException;

    void fsync() throws IOException;

    /**
     * Returns information about what was written to the journal
     *
     * @return A JournalSummary indicating what was written to the journal
     * @throws IOException if unable to write to the underlying storage mechanism.
     */
    JournalSummary getSummary();

    /**
     * @return <code>true</code> if the journal is healthy and can be written to, <code>false</code> if either the journal has been closed or is poisoned
     */
    boolean isHealthy();

    /**
     * Destroys any resources that the journal occupies
     */
    void dispose();
}
