package org.xbib.event.bus;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.Executor;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertTrue;

public class AsyncEventBusTest {

    private static final String EVENT = "Hello";

    private FakeExecutor executor;

    private AsyncEventBus bus;

    @BeforeEach
    protected void setUp() {
        executor = new FakeExecutor();
        bus = new AsyncEventBus(executor);
    }

    @Test
    public void testBasicDistribution() {
        StringCatcher catcher = new StringCatcher();
        bus.register(catcher);

        // We post the event, but our Executor will not deliver it until instructed.
        bus.post(EVENT);

        List<String> events = catcher.getEvents();
        assertTrue(events.isEmpty(), "No events should be delivered synchronously.");

        // Now we find the task in our Executor and explicitly activate it.
        List<Runnable> tasks = executor.getTasks();
        assertEquals(1, tasks.size(), "One event dispatch task should be queued.");

        tasks.get(0).run();

        assertEquals(1, events.size(), "One event should be delivered.");
        assertEquals(EVENT, events.get(0), "Correct string should be delivered.");
    }

    /**
     * An {@link Executor} wanna-be that simply records the tasks it's given. Arguably the Worst
     * Executor Ever.
     */
    public static class FakeExecutor implements Executor {
        List<Runnable> tasks = new ArrayList<>();

        @Override
        public void execute(Runnable task) {
            tasks.add(task);
        }

        public List<Runnable> getTasks() {
            return tasks;
        }
    }
}
