package org.xbib.event.syslog;

import java.net.InetAddress;
import java.time.LocalDateTime;

/**
 * Interface represents an incoming syslog request. This interface acts as an intermediary between
 * the TCPSyslogMessageDecoder and UDPSyslogMessageDecoder
 *
 * @see TCPSyslogMessageDecoder
 * @see UDPSyslogMessageDecoder
 */
public interface SyslogRequest {
    /**
     * The time the message was received by Netty.
     *
     * @return The time the message was received by Netty.
     */
    LocalDateTime receivedDate();

    /**
     * IP Address for the sender of the message.
     *
     * @return Sender IP Address
     */
    InetAddress remoteAddress();

    /**
     * The raw message that was delivered
     *
     * @return Raw message.
     */
    String rawMessage();

    interface Builder {

        SyslogRequest build();
    }
}
